// stores/room.ts - 管理房间状态
import { defineStore } from 'pinia'
import { ref, reactive, computed } from 'vue'
import { roomService } from '@/services/room'
import { 
  type Room, 
  type RoomUserInfo, 
  type RecentRoom, 
  RoomRole, 
  RoomStatus, 
  RoomVisibility 
} from '@/types/room'
import { useUserStore } from '@/stores/user'
import Taro from '@tarojs/taro'

export const useRoomStore = defineStore('room', () => {
  // 获取userStore
  const userStore = useUserStore()
  
  // 当前房间信息
  const currentRoom = ref<Room | null>(null)
  
  // 最近加入的房间
  const recentRooms = ref<RecentRoom[]>([])
  
  // 历史记录状态
  const historyState = reactive({
    activeRooms: [] as Room[],
    endedGames: [] as any[],
    isLoading: false
  })
  
  // 设置当前房间
  function setCurrentRoom(room: Room | null) {
    currentRoom.value = room
  }
  
  // 加载最近房间列表 - 使用统一的数据源
  async function loadRecentRooms() {
    // 首先确保活跃房间已加载 
    if (historyState.activeRooms.length === 0) {
      await loadActiveRooms()
    }
    
    // 从已加载的活跃房间和历史记录构建最近房间列表
    const recentList: RecentRoom[] = [
      // 活跃房间
      ...historyState.activeRooms.map(room => ({
        id: room.id,
        name: room.name,
        gameTitle: room.gameTitle,
        lastVisitTime: room.createTime,
        hasPassword: room.visibility === RoomVisibility.PRIVATE,
        status: room.status,
        playerCount: room.users.length
      })),
      // 已结束游戏
      ...historyState.endedGames.filter((game, index) => index < 5).map(game => ({
        id: game.roomId || game.id,
        name: game.roomName,
        gameTitle: game.gameTitle,
        lastVisitTime: game.endTime,
        status: RoomStatus.ENDED,
        hasPassword: false,
        playerCount: game.users?.length || 0
      }))
    ]
    
    // 按时间排序并只保留最近的条目
    recentRooms.value = recentList
      .sort((a, b) => b.lastVisitTime - a.lastVisitTime)
      .slice(0, 10)
    
    return recentRooms.value
  }
  
  // 加载活跃房间 - 统一数据源
  async function loadActiveRooms() {
    try {
      historyState.activeRooms = await roomService.getActiveRooms(userStore.openid)
      return historyState.activeRooms
    } catch (error) {
      console.error('加载活跃房间失败:', error)
      return []
    }
  }
  
  // 加载已结束游戏 - 统一数据源
  async function loadEndedGames() {
    try {
      historyState.endedGames = await roomService.getEndedGames(userStore.openid)
      return historyState.endedGames
    } catch (error) {
      console.error('加载已结束游戏失败:', error)
      return []
    }
  }
  
  // 检查用户是否已注册，如果未注册则返回错误
  async function checkUserRegistered(): Promise<{valid: boolean, message?: string}> {
    if (!userStore.isRegistered) {
      return { 
        valid: false, 
        message: '需要先完善个人信息才能继续操作' 
      }
    }
    return { valid: true }
  }

  // 检查房间是否存在
  async function checkRoomExist(roomId: string) {
    try {
      return await roomService.checkRoomExist(roomId)
    } catch (error) {
      console.error('检查房间存在失败:', error)
      return { exists: false, needPassword: false }
    }
  }

  // 验证房间密码
  async function verifyRoomPassword(roomId: string, password: string) {
    try {
      return await roomService.verifyRoomPassword(roomId, password)
    } catch (error) {
      console.error('验证房间密码失败:', error)
      return { valid: false }
    }
  }
  
  // 创建房间 - 调用API并更新store
  async function createRoom(roomData: Room) {
    try {
      // 检查用户是否已注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { roomId: null, success: false, message: userCheck.message }
      }
      
    // 确保用户列表存在
    if (!roomData.users) {
      roomData.users = []
    }
    
    // 添加主持人用户到房间用户列表
    roomData.users.push({
      openid: userStore.openid,
      nickname: userStore.nickname,
      avatar: userStore.avatar,
      roomRole: RoomRole.HOSTER, // 在Store内部设置角色为主持人
      joinTime: Date.now(),
      isReady: true // 主持人默认已准备
    })
    
    // 设置主持人信息
    roomData.hosterId = userStore.openid
    
    const result = await roomService.createRoom(roomData, userStore)
    
    if (result.success && result.roomId) {
      // 获取完整房间信息
      const room = await roomService.getRoomInfo(result.roomId)
      if (room) {
        setCurrentRoom(room)
        
        // 设置当前房间ID
        userStore.setCurrentRoom(result.roomId)
        
        // 设置当前游戏ID
        if (room.gameId) {
          userStore.setCurrentGame(room.gameId)
        }
        
        // 更新最近房间列表
        await loadRecentRooms()
      }
    }
    
    return result
  } catch (error) {
    console.error('创建房间失败:', error)
    return { roomId: null, success: false, message: '创建房间失败' }
  }
  }
  
  // 加入房间 - 调用API并更新store
  async function joinRoom(roomId: string) {
    try {
      // 检查用户是否已注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { success: false, message: userCheck.message }
      }
      
    // 构建用户数据 - 内部设置角色为玩家
    const userData: RoomUserInfo = {
      openid: userStore.openid,
      nickname: userStore.nickname,
      avatar: userStore.avatar,
      roomRole: RoomRole.PLAYER, // 在Store内部设置角色为玩家
      joinTime: Date.now(),
      isReady: false
    }
      
      // 调用API加入房间
      const result = await roomService.joinRoom(roomId, userData)
      
      if (result.success && result.roomData) {
        // 更新当前房间
        setCurrentRoom(result.roomData)
        
        // 更新用户当前房间ID
        userStore.setCurrentRoom(roomId)
        
        // 记录当前游戏ID
        if (result.roomData.gameId) {
          userStore.setCurrentGame(result.roomData.gameId)
        }
        
        // 重新加载活跃房间和最近房间列表以保持数据一致性
        await loadActiveRooms()
        await loadRecentRooms()
      }
      
      return result
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
  
  // 退出房间 - 调用API并更新store
  async function leaveRoom(roomId: string) {
    try {
      const result = await roomService.leaveRoom(roomId, userStore.openid)
      
      if (result.success) {
        // 清除当前房间信息
        clearRoom()
        
        // 清除用户当前房间ID
        userStore.setCurrentRoom('')
      }
      
      return result
    } catch (error) {
      console.error('退出房间失败:', error)
      return { success: false, message: '退出房间失败' }
    }
  }
  
  // 加载房间信息 - 通过API获取
  async function loadRoomInfo(roomId: string) {
    try {
      const room = await roomService.getRoomInfo(roomId)
      
      if (room) {
        setCurrentRoom(room)
        return { success: true, room }
      }
      
      return { success: false, message: '房间不存在' }
    } catch (error) {
      console.error('加载房间信息失败:', error)
      return { success: false, message: '加载房间信息失败' }
    }
  }
  
  // 添加用户到当前房间
  function addUserToRoom(user: RoomUserInfo) {
    if (currentRoom.value) {
      // 检查是否已存在
      const existingUserIndex = currentRoom.value.users.findIndex(u => u.openid === user.openid)
      if (existingUserIndex !== -1) {
        // 更新现有用户信息
        currentRoom.value.users[existingUserIndex] = {
          ...currentRoom.value.users[existingUserIndex],
          ...user
        }
      } else {
        // 添加新用户
        currentRoom.value.users.push(user)
      }
    }
  }
  
  // 从当前房间移除用户
  function removeUserFromRoom(userId: string) {
    if (currentRoom.value) {
      currentRoom.value.users = currentRoom.value.users.filter(u => u.openid !== userId)
    }
  }
  
  // 更新用户信息
  async function updateUserInRoom(userId: string, data: Partial<RoomUserInfo>) {
    if (!currentRoom.value) return { success: false, message: '当前没有加入房间' }
    
    try {
      // 调用API更新用户信息
      const result = await roomService.updateUserInRoom(currentRoom.value.id, userId, data)
      
      if (result.success) {
        // 更新本地状态
        const userIndex = currentRoom.value.users.findIndex(u => u.openid === userId)
        if (userIndex !== -1) {
          currentRoom.value.users[userIndex] = {
            ...currentRoom.value.users[userIndex],
            ...data
          }
        }
        return result
      } else {
        return result
      }
    } catch (error) {
      console.error('更新用户信息失败:', error)
      return { success: false, message: '更新用户信息失败' }
    }
  }
  
  // 更新房间状态
  async function updateRoomStatus(status: RoomStatus) {
    if (!currentRoom.value) return { success: false, message: '当前没有加入房间' }
    
    try {
      // 调用API更新房间状态
      const result = await roomService.updateRoomStatus(currentRoom.value.id, status)
      
      if (result.success) {
        // 更新本地状态
        currentRoom.value.status = status
        return result
      } else {
        return result
      }
    } catch (error) {
      console.error('更新房间状态失败:', error)
      return { success: false, message: '更新房间状态失败' }
    }
  }
  
  // 清除当前房间
  function clearRoom() {
    currentRoom.value = null
  }
  
  // 辅助方法：获取当前房间用户数量
  function getUserCount() {
    return currentRoom.value?.users.length || 0
  }
  
  // 辅助方法：检查房间是否已满
  function isRoomFull() {
    if (!currentRoom.value) return false
    return currentRoom.value.users.length >= currentRoom.value.maxPlayers
  }
  
  // 辅助方法：获取主持人信息
  function getHoster() {
    if (!currentRoom.value) return null
    return currentRoom.value.users.find(u => u.roomRole === RoomRole.HOSTER) || null
  }
  
  // 辅助方法：获取玩家列表（不包括主持人）
  function getPlayers() {
    if (!currentRoom.value) return []
    return currentRoom.value.users.filter(u => u.roomRole === RoomRole.PLAYER)
  }
  
  // 检查用户是否在房间中
  function isUserInRoom(userId: string) {
    if (!currentRoom.value) return false
    return currentRoom.value.users.some(u => u.openid === userId)
  }
  
  // 获取用户在房间中的角色
  function getUserRole(userId: string): RoomRole | null {
    if (!currentRoom.value) return null
    const user = currentRoom.value.users.find(u => u.openid === userId)
    return user ? user.roomRole : null
  }
  
  // 更新房间设置
  function updateRoomSettings(settings: Partial<Room>) {
    if (currentRoom.value) {
      currentRoom.value = {
        ...currentRoom.value,
        ...settings
      }
    }
  }
  
  // 加载历史记录 - 修改为使用共享数据源
  async function loadHistoryByTab(tabIndex: number) {
    historyState.isLoading = true
    
    try {
      if (tabIndex === 0) { // 最近游戏
        // 加载进行中的房间和已结束的游戏
        await loadActiveRooms()
        await loadEndedGames()
        
      } else if (tabIndex === 1) { // 我创建的
        // 确保数据已加载
        await loadActiveRooms()
        await loadEndedGames()
        
        // 筛选我创建的内容
        historyState.activeRooms = historyState.activeRooms.filter(
          room => room.hosterId === userStore.openid
        )
        historyState.endedGames = historyState.endedGames.filter(
          game => game.hosterId === userStore.openid
        )
        
      } else if (tabIndex === 2) { // 我参与的
        // 确保数据已加载
        await loadActiveRooms()
        await loadEndedGames()
        
        // 筛选我参与但不是我创建的内容
        historyState.activeRooms = historyState.activeRooms.filter(
          room => room.hosterId !== userStore.openid && 
                room.users && 
                room.users.some(u => u.openid === userStore.openid)
        )
        historyState.endedGames = historyState.endedGames.filter(
          game => game.hosterId !== userStore.openid &&
                game.users && 
                game.users.some(u => u.openid === userStore.openid)
        )
      }
      
      // 每次加载历史记录后更新最近房间列表
      await loadRecentRooms()
    } catch (error) {
      console.error('加载历史记录失败:', error)
    } finally {
      historyState.isLoading = false
    }
  }
  
  // 为join页面提供的限制条数的计算属性
  const limitedRecentRooms = computed(() => {
    return recentRooms.value.slice(0, 2) // 只显示最近2条
  })
  
  // 修改 joinRoomById 方法，确保返回房间数据
  async function joinRoomById(roomId: string, password?: string): Promise<{
    success: boolean, 
    message?: string, 
    needPassword?: boolean,
    roomData?: Room  // 添加房间数据
  }> {
    try {
      // 1. 检查用户是否注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { success: false, message: userCheck.message }
      }
      
      // 2. 验证房间
      const checkResult = await roomService.checkRoomExist(roomId)
      if (!checkResult || !checkResult.exists) {
        return { success: false, message: '房间不存在' }
      }
      
      // 3. 验证密码
      if (checkResult.needPassword && !password) {
        return { success: false, message: '需要密码', needPassword: true }
      }
      
      if (checkResult.needPassword && password) {
        const pwdResult = await roomService.verifyRoomPassword(roomId, password)
        if (!pwdResult || !pwdResult.valid) {
          return { success: false, message: '房间密码错误' }
        }
      }
      
      // 4. 加入房间
      const joinResult = await joinRoom(roomId)
      
      // 返回结果时同时包含房间数据
      return joinResult
      
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
  
  // 添加一个根据房间状态执行跳转的辅助方法
  function navigateToRoomPage(roomId: string) {
    // 获取当前房间状态
    const status = currentRoom.value?.status
    
    if (status === RoomStatus.PLAYING) {
      // 如果房间状态为"进行中"，跳转到游戏页面
      Taro.navigateTo({
        url: `/pages/room/play/index?roomId=${roomId}`
      })
    } else {
      // 如果房间状态为"等待中"或其他状态，跳转到等待页面
      Taro.navigateTo({
        url: `/pages/room/waiting/index?roomId=${roomId}`
      })
    }
  }
  
  // 返回状态和方法
  return {
    // 状态
    currentRoom,
    recentRooms,
    limitedRecentRooms,
    historyState,
    
    // 房间操作方法
    setCurrentRoom,
    checkRoomExist,
    verifyRoomPassword,
    createRoom,
    joinRoom,
    leaveRoom,
    loadRoomInfo,
    loadRecentRooms,
    loadActiveRooms,
    loadEndedGames,
    clearRoom,
    
    // 用户管理方法
    addUserToRoom,
    removeUserFromRoom,
    updateUserInRoom,
    isUserInRoom,
    getUserRole,
    
    // 房间属性方法
    updateRoomStatus,
    updateRoomSettings,
    
    // 辅助方法
    getUserCount,
    isRoomFull,
    getHoster,
    getPlayers,
    
    // 历史记录方法
    loadHistoryByTab,
    joinRoomById,
    navigateToRoomPage,
  }
})